/*
 * FILE:   video_display.c
 * AUTHOR: Colin Perkins <csp@isi.edu>
 *
 * Copyright (c) 2001-2003 University of Southern California
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * 
 *      This product includes software developed by the University of Southern
 *      California Information Sciences Institute.
 * 
 * 4. Neither the name of the University nor of the Institute may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Revision: 1.6 $
 * $Date: 2003/09/01 15:14:18 $
 *
 */

#include "config.h"
#include "config_unix.h"
#include "config_win32.h"
#include "debug.h"
#include "video_display.h"
#include "video_display/null.h"
#include "video_display/xv.h"
#include "video_display/x11.h"
#include "video_display/hdstation.h"

/*
 * Interface to probing the valid display types. 
 *
 */

typedef struct {
	display_id_t		 id;
	display_type_t 		*(*func_probe)(void);
	void			*(*func_init)(void);
	void 			 (*func_done)(void *state);
	char 			*(*func_getf)(void *state);
	int  			 (*func_putf)(void *state, char *frame);
	display_colour_t	 (*func_colour)(void *state);
} display_table_t;

static display_table_t display_device_table[] = {
#ifndef X_DISPLAY_MISSING
	{
		0,
		display_xv_probe,
		display_xv_init,
		display_xv_done,
		display_xv_getf,
		display_xv_putf,
		display_xv_colour
	},
	{
		0,
		display_x11_probe,
		display_x11_init,
		display_x11_done,
		display_x11_getf,
		display_x11_putf,
		display_x11_colour
	},
#endif
#ifdef HAVE_HDSTATION
	{
		0,
		display_hdstation_probe,
		display_hdstation_init,
		display_hdstation_done,
		display_hdstation_getf,
		display_hdstation_putf,
		display_hdstation_colour
	},
#endif
	{
		0,
		display_null_probe,
		display_null_init,
		display_null_done,
		display_null_getf,
		display_null_putf,
		display_null_colour
	}
};

#define DISPLAY_DEVICE_TABLE_SIZE (sizeof(display_device_table) / sizeof(display_table_t))

static display_type_t	*available_devices[DISPLAY_DEVICE_TABLE_SIZE];
static int		 available_device_count = 0;

int
display_init_devices(void)
{
	unsigned int	 i;
	display_type_t	*dt;

	assert(available_device_count == 0);

	for (i = 0; i < DISPLAY_DEVICE_TABLE_SIZE; i++) {
		dt = display_device_table[i].func_probe();
		if (dt != NULL) {
			display_device_table[i].id = dt->id;
			available_devices[available_device_count++] = dt;
		}
	}
	return 0;
}

void 
display_free_devices(void)
{
	int	i;

	for (i = 0; i < available_device_count; i++) {
		free(available_devices[i]);
		available_devices[i] = NULL;
	}
	available_device_count = 0;
}

int
display_get_device_count(void)
{
	return available_device_count;
}

display_type_t *
display_get_device_details(int index)
{
	assert(index < available_device_count);
	assert(available_devices[index] != NULL);

	return available_devices[index];
}

display_id_t
display_get_null_device_id(void)
{
	return DISPLAY_NULL_ID;
}

/*
 * Display initialisation and playout routines...
 */

#define DISPLAY_MAGIC 0x01ba7ef1

struct display {
	uint32_t	 magic;
	int		 index;
	void		*state;
};

struct display *
display_init(display_id_t id)
{
	unsigned int	i;

	for (i = 0; i < DISPLAY_DEVICE_TABLE_SIZE; i++) {
		if (display_device_table[i].id == id) {
			struct display *d = (struct display *) malloc(sizeof(struct display));
			d->magic = DISPLAY_MAGIC;
			d->state = display_device_table[i].func_init();
			d->index = i;
			if (d->state == NULL) {
				debug_msg("Unable to start display 0x%08lx\n", id);
				free(d);
				return NULL;
			} 
			return d;
		}
	}
	debug_msg("Unknown display id: 0x%08x\n", id);
	return NULL;
}

void
display_done(struct display *d)
{
	assert(d->magic == DISPLAY_MAGIC);
	display_device_table[d->index].func_done(d->state);
}

char *
display_get_frame(struct display *d)
{
	assert(d->magic == DISPLAY_MAGIC);
	return display_device_table[d->index].func_getf(d->state);
}

void
display_put_frame(struct display *d, char *frame)
{
	assert(d->magic == DISPLAY_MAGIC);
	display_device_table[d->index].func_putf(d->state, frame);
}

display_colour_t 
display_get_colour_mode(struct display *d)
{
	assert(d->magic == DISPLAY_MAGIC);
	return display_device_table[d->index].func_colour(d->state);
}

